/*
 * Copyright (c) 1994 Frank Behrens
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*  $Log: rcpos2.c,v $
 * Revision 1.2  1996/11/19  22:22:52  frank
 * display filenames, size and percent
 * h_error
 *
 * Revision 1.1  1994/09/09  00:00:00  frank
 * Initial revision
 *                                                       */

static const char rcsid[] = "$Id: rcpos2.c,v 1.2 1996/11/19 22:22:52 frank Exp $";


#include "rcpos2.h"
#include <sys/types.h>
#include <netdb.h>
#include <pwd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#define INCL_DOSSEMAPHORES
#define INCL_DOSPROCESS
#include <os2.h>

static
char    *h_errlist[] = {
        "Error 0",
        "Unknown host",                         /* 1 HOST_NOT_FOUND */
        "Host name lookup failure",             /* 2 TRY_AGAIN */
        "Unknown server error",                 /* 3 NO_RECOVERY */
        "No address associated with name",      /* 4 NO_ADDRESS */
        };

static
int     h_nerr = { sizeof(h_errlist)/sizeof(h_errlist[0]) };


/* Handle for Semaphore */
static HEV sema;

/* Handle for Thread */
static TID thid;

static long curr_bytes;
/******************************************************************/

struct passwd *getpwuid (int uid)

{
static struct passwd pw;
char *user;

memset (&pw, 0, sizeof (pw));

user = getenv ("USER");

pw.pw_name   = user ? strdup (user) : "os2user";
pw.pw_passwd = strdup ("");

return &pw;

}


int setuid (uid_t uid)

{
return 0;
}

int seteuid (uid_t uid)

{
return 0;
}


void convert_pathdelim (char *path, int to_dos)

{
while (*path) {
   if (to_dos)
      { if (*path == '/') *path = '\\'; }
   else
      { if (*path == '\\') *path = '/'; }
   path++;
   }
}


void alarmhandler (void)

{
exit (0);
}


void herror (char *msg)

{
int h_err;
h_err = h_errno;

/* bug in IBM's TCP/IP ? */
if (! h_err)
   h_err = 1;
/* bug in IBM's TCP/IP ? */

if (msg)
   fprintf (stderr, "%s: ", msg);

fprintf (stderr, "%s\n", h_err < h_nerr ?
         h_errlist[h_err] : "Unknown error");

}



void create_semaphore (void)

{
APIRET rc;

rc = DosCreateEventSem (NULL, &sema, 0L, FALSE /* reset */);
if (rc) {
   fprintf (stderr, "ERROR creating semaphore #%li\n", rc);
   exit (100);
   }
}

void delete_semaphore (void)

{
DosCloseEventSem (sema);
sema = 0;
}



void show_percent (void *dummy)

{
APIRET rc;
ULONG  PostCtr;

while (1) {
   if (curr_bytes != bytes_curr) {
      curr_bytes = bytes_curr;
      if (bytes_all)  /* avoid division by 0 */
         printf ("%3li%%\b\b\b\b", bytes_curr * 100 / bytes_all);
      }
   if (bytes_curr >= bytes_all)
      break;
   rc = DosWaitEventSem (sema, 1000);
   if (rc != 640)
      break;
   DosResetEventSem (sema, &PostCtr);
   }
}



void start_showthread (int verbose)

{
ULONG  PostCtr;

if (! verbose)
   return;
curr_bytes = -1;
DosResetEventSem (sema, &PostCtr);
thid = _beginthread (show_percent, NULL, 16000, NULL);
if (thid == -1) {
   fprintf (stderr, "ERROR creating thread\n");
   exit (100);
   }
}



void end_showthread (int verbose)

{
if (! verbose)
   return;

if (thid) {
   DosPostEventSem (sema);
   DosWaitThread (&thid, DCWW_WAIT);
   thid = 0;
   }
}

char * quiet_4os2_switch (int verbose)

{
int set_q = 0;
char *pcomspec, comspec[1024];

comspec[0] = '\0';
if (! verbose) {  /* nur wenn nichts angezeigt werden soll */
   pcomspec = getenv ("COMSPEC");
   if (pcomspec)
      strcpy (comspec, pcomspec);
   strlwr (comspec);
   if (strstr (comspec, "4os2"))
      set_q++;
   }
return set_q ? "/q " : "";
}
